from django.shortcuts import get_object_or_404, render, redirect
from django.http import HttpResponse, HttpResponseRedirect
from django.urls import reverse, reverse_lazy
from django.contrib import messages
from django.contrib.auth import update_session_auth_hash, authenticate, login, logout
from django.contrib.auth.forms import PasswordChangeForm, UserCreationForm
from django.contrib.auth.models import User
from django.utils.translation import gettext as _
from django.utils import timezone, dateformat


#User operations form imports
from .assets.forms import UserForm, ProfileForm, LoginForm
from django.contrib.auth.decorators import login_required
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.views import PasswordChangeView
from django.db import transaction
from django.views.decorators.csrf import csrf_exempt


#Models used and forms imported
from .models import Estoque, FAQ, Loja, Imagemestoque, Cart, Pedido, Ordem, Tag, Frete, Pagamento
from django import forms

#Include extarnal functions  to create advanced context for templates
from .assets.contextFunctions import generate_template_header_context

#query para filter
from django.db.models import Q

#paginação
from django.core.paginator import Paginator

#Normal Python libraries
from datetime import date
import os
import json

#Send email
from django.core.mail import send_mail
from django.template.loader import render_to_string
from django.utils.html import strip_tags

#Import Payment object
from .assets.asaaspayment import Asaas

#useful functions
from erp.assets.networkfunctions import add_days, handle_file_upload,buscar_informacoes,ler_csv

#save file mass import
from django.core.files import File

#custom user form:
@login_required
@transaction.atomic
def update_profile(request):
    if request.method == 'POST':
        user_form = UserForm(request.POST, instance=request.user)
        profile_form = ProfileForm(request.POST, request.FILES, instance=request.user.profile)
        if user_form.is_valid() and profile_form.is_valid():
            user_form.save()
            profile_form.save()
            messages.success(request, _('Your profile was successfully updated!'))
            return redirect(inicio)
        else:
            messages.error(request, _('Please correct the error below.'))
    else:
        user_form = UserForm(instance=request.user)
        profile_form = ProfileForm(instance=request.user.profile)
    
    header_context = generate_template_header_context(request)
    context = {
        "header_context" : header_context,
        'user_form': user_form,
        'profile_form': profile_form,
    }
    
    return render(request, 'erp/update-profile-form.html', context)
    

class PasswordsChangeView(PasswordChangeView):
    form_class = PasswordChangeForm#, template_name='erp/change-password.html')
    success_url = reverse_lazy("inicio")
    

def inicio(request):
    linha1 = []
    linha2 = []
    path=os.path.join('erp','static','img','carrossel')
    for filename in os.listdir(path):
        if 'line_1' in filename: 
            linha1.append(f'img/carrossel/{filename}')
        else:
            linha2.append(f'img/carrossel/{filename}')
    header_context = generate_template_header_context(request)
    newest_games= Estoque.objects.order_by('-dtcadastro')[:10]
    context = {
        "header_context" : header_context,
        "newest_games" : newest_games,
        "linha1" : linha1,
        "linha2" : linha2,
    }
    return render(request, 'erp/inicio.html', context)


def comprar(request):
    header_context = generate_template_header_context(request)
    context = {
        "header_context" : header_context,
    }
    return render(request,'erp/comprar.html', context)

def login_area(request):
    if request.method=="POST":
        username=request.POST['username']
        password=request.POST['password']
        user=authenticate(username=username,password=password)
        if user is not None:
            login(request,user)
            messages.success(request,("Login com Sucesso"))
            return redirect('inicio')
        else:
            messages.error(request,("Erro ao fazer login"))
            return redirect('login')
    else:
        header_context = generate_template_header_context(request)
        user_form = LoginForm()
        context = {
            "header_context" : header_context,
            "form" : user_form,
        }
        return render(request, 'erp/login.html', context)

def criar_conta(request):
    form = UserCreationForm()
    if request.method == "POST":
        form = UserCreationForm(request.POST)
        if form.is_valid():
            form.save()
            username = form.cleaned_data['username']
            password1 = form.cleaned_data['password1']
            password2 = form.cleaned_data['password2']
            if password1==password2:
                user = authenticate(username=username, password=password1)
                login(request,user)
                return redirect('criar_conta_passo2')
            else:
                messages.error('Senhas diferentes')
                return redirect('criar_conta')
        else:
            return redirect('criar_conta')
    else:
        #loja_data = Loja.objects.first()
        header_context = generate_template_header_context(request)
        context = {
            "header_context" : header_context,
            'form': form,
        }
        return render(request, 'erp/criar-conta.html', context)
            #{'form' : form, 'loja_data': loja_data})

@login_required
@transaction.atomic
def criar_conta_passo2(request):
    if request.method == 'POST':
        user_form = UserForm(request.POST, instance=request.user)
        profile_form = ProfileForm(request.POST, request.FILES, instance=request.user.profile)
        if user_form.is_valid() and profile_form.is_valid():
            user_form.save()
            profile_form.save()
            messages.success(request, _('Your profile was successfully updated!'))
            return redirect(inicio)
        else:
            messages.error(request, _('Please correct the error below.'))
    else:
        user_form = UserForm(instance=request.user)
        profile_form = ProfileForm(instance=request.user.profile)
    header_context = generate_template_header_context(request)
    context = {
        "header_context" : header_context,
        'user_form': user_form,
        'profile_form': profile_form,
        }
    return render(request, 'erp/criar-conta-passo2.html', context)
    
    

def duvidas(request):
    duvidas_data = FAQ.objects.all()
    header_context = generate_template_header_context(request)
    context = {
        "header_context" : header_context,
        "duvidas": duvidas_data,
    }
    return render(request,'erp/duvidas.html', context)

def logout_area(request):
    logout(request)
    messages.success(request,("Desconectado com Sucesso"))
    #loja_data=Loja.objects.first()
    return redirect('inicio')

#Cart Management views
@login_required
def add_to_cart(request, product_id):
    cart_item = Cart.objects.filter(user=request.user, product__pk=product_id).first()
    jogo = Estoque.objects.get(pk=product_id)
    if cart_item:
        cart_item.quantity += 1
        cart_item.save()
        messages.success(request, f"{jogo.nome} adicionado ao seu carrinho.")
    else:
        Cart.objects.create(user=request.user, product=jogo)
        messages.success(request, f"{jogo.nome} adicionado ao seu carrinho.")
    source = request.META.get('HTTP_REFERER')
    if source:
        return redirect(source)
    else:
        #In case the referer site is strange
        return redirect("cart_detail")

@login_required
def remove_from_cart(request, cart_item_id):
    cart_item = get_object_or_404(Cart, id=cart_item_id)
    jogo = Estoque.objects.get(pk=cart_item.product.pk)
    if cart_item.user == request.user:
        if cart_item.quantity <= 1:
            cart_item.delete()
            messages.success(request, f"{jogo.nome} removido do seu carrinho.")
        else:
            cart_item.quantity -= 1
            cart_item.save()
            messages.success(request, f"{jogo.nome} removido do seu carrinho.")

    return redirect("cart_detail")

@login_required
def cart_detail(request):
    cart_items = Cart.objects.filter(user=request.user)
    itens_no_cart = cart_items.count()
    total_price = sum(item.quantity * item.product.valor for item in cart_items)
    header_context = generate_template_header_context(request)
    #loja_data=Loja.objects.first()
    #Aqui precisamos passar o item no produto tb para podermos ter acesso a info dentro do
    #HTML para fazer o display
    context = {
        "cart_items": cart_items,
        "total_price": total_price,
        "header_context" : header_context,
    }
    return render(request, "erp/carrinho.html", context)



#View dos jogos com detalhes
def jogo(request,pk):
    jogo=Estoque.objects.prefetch_related('imagemestoque_set').get(id=pk)
    fotos=Imagemestoque.objects.filter(produto=pk)
    header_context = generate_template_header_context(request)
    context = {
        "fotos": fotos,
        "header_context" : header_context,
        "jogo": jogo,
    }
    return render(request,'erp/jogo.html', context)

def alugar(request):
    #rafael
    lookup=None
    search_values = {}
    #Pa garantir que nao entra com nome do jogo estranho em caso de Post vazio.
    name = None
    #Verifica se vem uma search nova via post, pega e cria o filtro e popula o search_values
    if request.method=="POST":
        name = request.POST.get('name', '')
        min_price = request.POST.get('min-price', '')
        max_price = request.POST.get('max-price', '')
        num_players = request.POST.get('num-players', '')
        status = request.POST.get('status', '')
        tag = request.POST.get('tag','')
        lookup=Q()
        if name:
            lookup&=Q(nome__icontains=name) 
            search_values["name"] = name
        if min_price:
            lookup&=Q(valor__gte=min_price)
            search_values["min_price"] = min_price
        if max_price:
            lookup&=Q(valor__lte=max_price)
            search_values["max_price"] = max_price
        if num_players:
            lookup&=Q(minjogadores__lte=num_players)&Q(maxjogadores__gte=num_players)
            search_values["num_players"] = num_players
        if status:
            search_values["status"] = status
            if status=="disponivel":
                lookup&=Q(disponivel__gte=1)
            elif status=="alugado":
                lookup&=Q(disponivel__lte=0)
        if tag:
            if tag != "todos":
                search_values["tag"] = tag
                lookup&=Q(tags__tag__icontains=tag)
  
    #Ou Se for link paginado pega os parametros via Get e passa para search_values
    if request.method=="GET":
        name = request.GET.get('name')
        min_price = request.GET.get('min-price')
        max_price = request.GET.get('max-price')
        num_players = request.GET.get('num-players')
        status = request.GET.get('status', '')
        tag = request.GET.get('tag','')

        lookup=Q()
        if name:
            lookup&=Q(nome__icontains=name) 
            search_values["name"] = name
        if min_price:
            lookup&=Q(valor__gte=min_price)
            search_values["min_price"] = min_price
        if max_price:
            lookup&=Q(valor__lte=max_price)
            search_values["max_price"] = max_price
        if num_players:
            lookup&=Q(minjogadores__lte=num_players)&Q(maxjogadores__gte=num_players)
            search_values["num_players"] = num_players
        if status:
            search_values["status"] = status
            if status=="disponivel":
                lookup&=Q(disponivel__gte=1)
            elif status=="alugado":
                lookup&=Q(disponivel__lte=0)
        if tag:
            if tag != "todos":
                search_values["tag"] = tag
                lookup&=Q(tags__tag__icontains=tag)

    #Actual flitering, for both Get or Post      
    if len(lookup) != 0:
        collapsado = False
        estoque_list = Estoque.objects.prefetch_related('imagemestoque_set').filter(lookup)
        paginacao=Paginator(Estoque.objects.prefetch_related('imagemestoque_set').filter(lookup),28)
    else:
        collapsado = True
        estoque_list = Estoque.objects.prefetch_related('imagemestoque_set').all()
        paginacao=Paginator(Estoque.objects.prefetch_related('imagemestoque_set').all(),28)


    pagina=request.GET.get('page')
    estoque=paginacao.get_page(pagina)
    pagina_range=range(1,paginacao.num_pages+1)
    
    #collapsado = False

    #andre
    header_context = generate_template_header_context(request)
    
    if name==None:
        search_values["name"]=""
         
    tag_list = Tag.objects.all()
    
    context = {
        "header_context" : header_context,
        'estoque': estoque,
        'estoque_list': estoque_list,
        'pagina_range': pagina_range,
        'search_values': search_values,
        'tag_list': tag_list,
        'collapsado' : collapsado,
    }
    return render(request,'erp/alugar.html', context)

@login_required
def perfil(request,user_key):
    historico_pedidos= Pedido.objects.filter(cliente=user_key).order_by('-dataenvio')
    list_of_ids = []
    for item in historico_pedidos:
        list_of_ids.append(item.pk)
    historico_jogos = Ordem.objects.filter(pedido__in=list_of_ids)
            
    header_context = generate_template_header_context(request)
    context = {
        "header_context" : header_context,
        "historico_pedidos":historico_pedidos,
        "historico_jogos": historico_jogos,
        #"list_fotos": list_fotos,
    }
    return render(request,'erp/perfil.html', context)


#views para completar ordem
#Modificar o carrinho no pedido
@login_required
def add_to_pedido(request, product_id):
    cart_item = Cart.objects.filter(user=request.user, product__pk=product_id).first()
    jogo = Estoque.objects.get(pk=product_id)
    if cart_item:
        cart_item.quantity += 1
        cart_item.save()
        messages.success(request, "Item added to your cart.")
    else:
        Cart.objects.create(user=request.user, product=jogo)
        messages.success(request, "Item added to your cart.")

    return redirect("complete_order")

@login_required
def remove_from_pedido(request, cart_item_id):
    cart_item = get_object_or_404(Cart, id=cart_item_id)

    if cart_item.user == request.user:
        if cart_item.quantity <= 1:
            cart_item.delete()
            messages.success(request, "Item removed from your cart.")
        else:
            cart_item.quantity -= 1
            cart_item.save()
            messages.success(request, "Item quantity reduced in cart.")

    return redirect("complete_order")

@login_required
def complete_order(request):
    header_context = generate_template_header_context(request)
    cart_items = Cart.objects.filter(user=request.user)
    preco_jogos = sum(item.quantity * item.product.valor for item in cart_items)
    context = {
        "header_context" : header_context,
        "cart_items" : cart_items,
        "preco_jogos" : preco_jogos,
    }
    return render(request,'erp/completar-pedido.html', context)

#This is a simple api that return the frete in a json file.
@login_required
def calcular_frete(request):
    if request.method=="GET":
        estado = request.GET.get('estado')
        cidade = request.GET.get('cidade')
        bairro = request.GET.get('bairro')
        frete_set = Frete.objects.values_list("valor").filter(estado=estado, cidade=cidade, bairro=bairro)
        if frete_set:
             for item in frete_set:
                data = {float(item[0])}
            
        else:
            #When there is no entry for the frete with the parameters
            data = {9999.0}
    else:
        #Alguem chegou aqui do jeito errado. For debug
        data = {9998.0}
    #Make sure make it URL safe. i.e space = %20 in JS.
    return HttpResponse(data, content_type='application/json')

#View para processar o pedido e criar pagamento
@login_required
def pedido_finalizado(request):
    if request.method == "POST":
        #We will always need this
        lojaData = Loja.objects.first()
        #messages.success(request, "Pedido feito")
        #criar novo pedido no db
        pedido = Pedido()
        pedido.cliente = request.user
        pedido.tipoentrega = request.POST.get('frete', 'Pickup')
        pedido.notas = request.POST.get('notas', '')
        pedido.endereco = request.POST.get('endereco','')
        pedido.complemento = request.POST.get('complemento','')
        pedido.bairro = request.POST.get('bairro','')
        pedido.cidade = request.POST.get('cidade','')
        pedido.estado = request.POST.get('estado','')
        pedido.cep = request.POST.get('cep','')
        pedido.dataenvio = request.POST.get('dataenvio', '')
        pedido.formapagamento = request.POST.get('pagamento', 'ERROR')
        pedido.save()
        # Adicionar itens do carrinho ao DB
        #Nao limpamos o cart ainda
        cart_items = Cart.objects.filter(user=request.user) 
        for item in cart_items:
            ordem_item = Ordem()
            ordem_item.produto = item.product
            ordem_item.quantidade = item.quantity
            ordem_item.pedido = pedido
            ordem_item.save()
        #Gerenciar o pagamento no Asaas em caso de ser PIX, BOLETO, CREDIT_CARD
        if pedido.formapagamento != 'CASH':
            #lojaData = Loja.objects.first()
            asaasConnection = Asaas(lojaData.asaastoken, production = False)
            #Verifica se cliente precisa ser registrado no Banco
            if request.user.profile.asaasid == '0':
                #Usuario ainda nao tem client id do asaas.
                newid = asaasConnection.create_client(request.user)
                if 'error' not in newid:
                    request.user.profile.asaasid = newid
                    request.user.save()
                else:
                    #Teve erro na criacao do cliente no banco. Nao modificamos nada e so rotear para erro.
                    header_context = generate_template_header_context(request)
                    context = {
                        "header_context" : header_context,
                    }
                    return render(request,'erp/erro-pedido.html', context)
                    
                    
            else: 
                newid = request.user.profile.asaasid

            #Actually create payment in the bank
            newpagamento = asaasConnection.nova_cobranca(request, pedido)
            if 'error' in str(newpagamento) : #Como newpagamento pode ser um objeto ou uma string, precisei converter para testar
                    #voltar para pagina de erro no banco
                    #Teve erro na criacao do pagamento, entao remove o pedido e vamos pedir para aguardar e colocar dfe novo
                    pedido.delete()
                    header_context = generate_template_header_context(request)
                    context = {
                        "header_context" : header_context,
                    }
                    return render(request,'erp/erro-pedido.html', context)             
        else:
            #Precisa fazer algo em caso de cash?
            #Acho q nao. Mas set variavel para nao dar erro
            newpagamento = 'NOBANK'
            newid = request.user.profile.asaasid


        #Montar o email para enviar para o cliente com a confirmacao do pedido
        #Basicamente usa o render para gerar o html em uma string.
        
        #Notifica cliente da criacao do pedido
        #ajustar para o smtp da producao! no settings.py
        if pedido.formapagamento != 'CASH':
            header_context = generate_template_header_context(request)
            cart_items = Cart.objects.filter(user=request.user) #not sure I need it here.. maybe for display
            context = {
                "header_context" : header_context,
                "cart_items" : cart_items,
                "pedido" : pedido,
                'pagamento' : newpagamento,
            }  
            message = render_to_string('erp/email-pedido-template.html', context)

            plain_message = strip_tags(message)
            subject = f'{lojaData.nome} - Pedido {pedido.pk} criado.'
            #send_mail(subject, plain_message, 'django@castelorj.com', [request.user.email], html_message=message )
            send_mail(subject, plain_message, str(lojaData.email), [request.user.email], html_message=message )
            if lojaData.notificapedido:
                subject = f'{pedido.cliente.email} - Pedido {pedido.pk} criado.'
                send_mail(subject, plain_message, str(lojaData.email), str(lojaData.email), html_message=message )
            messages.success(request, f"Pedido {pedido.pk} submetido:") 
        else:
            #Mesma mensagem so q sem referencia a pagamento, e dizendo que e retirada em maos
            header_context = generate_template_header_context(request)
            cart_items = Cart.objects.filter(user=request.user) #not sure I need it here.. maybe for display
            context = {
                "header_context" : header_context,
                "cart_items" : cart_items,
                "pedido" : pedido,
            }  
            message = render_to_string('erp/email-pedido-template-cash.html', context)

            plain_message = strip_tags(message)
            subject = f'{lojaData.nome} - Pedido {pedido.pk} criado.'
            #send_mail(subject, plain_message, 'django@castelorj.com',[request.user.email], html_message=message )
            send_mail(subject, plain_message, str(lojaData.email), [request.user.email], html_message=message )
            if lojaData.notificapedido:
                subject = f'{pedido.cliente.email} - Pedido {pedido.pk} criado.'
                send_mail(subject, plain_message, str(lojaData.email), str(lojaData.email), html_message=message )
            messages.success(request, f"Pedido {pedido.pk} submetido:") 


        #To make my life easier in the html, I will generate the data before deleting the cart'
        header_context = generate_template_header_context(request)
        header_context['itens_no_cart'] = 0 #Para mostrar q limpamos o cart apos o pedido na pagina target
        cart_items = Cart.objects.filter(user=request.user) #not sure I need it here.. maybe for display
        context = {
            "header_context" : header_context,
            "cart_items" : cart_items,
            "pedido" : pedido,
            'pagamento' : newpagamento,        }

        #Now that all worked, lets clean up the cart:
        for item in cart_items:
            item.delete() 
        
        #Finalmente mostra apagina do pedido      
        return render(request,'erp/pedido-finalizado.html', context)
        
    else:
        #messages.error(request, "Acesso indevido para gerar pedido")
        #voltar a pagina de fechar pedido pq chegou aqui de modo errado
        header_context = generate_template_header_context(request)
        cart_items = Cart.objects.filter(user=request.user)
        preco_jogos = sum(item.quantity * item.product.valor for item in cart_items)
        context = {
            "header_context" : header_context,
            "cart_items" : cart_items,
            "preco_jogos" : preco_jogos,
        }
        return render(request,'erp/completar-pedido.html', context)

@csrf_exempt
@transaction.atomic        
def retorno_bancario(request):
    #If somebody gets here with the wrong method (not post)
    if request.method != 'POST':
        data = {  
        "code":200,
        "status":"Success",
        "data":
            {  
                "descricao":"Weird  request method"
            }
        } 
        jsonValues = json.dumps(data)
        return HttpResponse(jsonValues, content_type='application/json')
    
    lojaData = Loja.objects.first()
    asaasConnection = Asaas(lojaData.asaastoken,lojaData.asaaswebhook, production = False)
    #This will update database and send a message that all is good or if is a duplicate event
    payload = json.loads(str(request.body)[2:-1]) # remove 1st char that for some reason is a b
    #payload = request.POST.dict()
    asaasConnection.handle_webhook(payload, request.headers.get('Asaas-Access-Token', '')) #The data is in the Body,  and not POST

    #return 200 to Asaas server if we get here. 
    #Should be really fast
    data = {  
    "code":200,
    "status":"success",
    "data":
        {  
            "descricao":"Evento processado com sucesso"
        }
    } 
    jsonValues = json.dumps(data, indent=4)
    
    return HttpResponse(jsonValues, content_type='application/json')
    
    #reports views


#Views dos reports
@staff_member_required
def reports(request):
    return render(request,'erp/reports.html')

@staff_member_required
def retornos(request):
    data = request.POST.get('data', dateformat.format(timezone.now(), 'Y-m-d'))
    ordens=Pedido.objects.filter(dataretorno = data, controlencerrado=False, status='Entregue')
    context = {
        "data" : data,
        "ordens" : ordens, 
    }
    return render(request,'erp/lista-retornos.html', context)
    
@staff_member_required
def entregas(request):
    #seleciona pedido
    #Organiza entregas por bairro um QS com o bairro como key e os pedidos como um QS
    #Pode loop por bairros, depois por pedido para mostrar agrupado
    data = request.POST.get('data', dateformat.format(timezone.now(), 'Y-m-d'))
    #Garante que so selecia bairros com pedidos para o dia
    value_list = Pedido.objects.values_list('bairro', flat=True).order_by('bairro').filter(status__in=['Pago', 'Pronto'], dataenvio=data).distinct()
    group_by_value = {}
    for value in value_list:
        group_by_value[value] = Pedido.objects.filter(bairro=value, status__in=['Pago', 'Pronto'], dataenvio=data)
    
    context = {
        "data": data, 
        "pedidos" : group_by_value, 
        "bairros" : value_list,
    }
    return render(request,'erp/lista-entregas.html', context)

@staff_member_required
def atrasados(request):
    data = request.POST.get('data', dateformat.format(add_days(days=-1), 'Y-m-d'))
    ordens=Pedido.objects.filter(dataretorno__range = ["2020-01-01", data], controlencerrado=False, status='Entregue')
    context = {
        "data" : data,
        "ordens" : ordens, 
    }
    return render(request,'erp/lista-atrasados.html', context)

@staff_member_required
def indisponiveis(request):
    QS=Estoque.objects.filter(disponivel=0)
    context = {
        "estoque" : QS, 
    }
    return render(request,'erp/lista-indisponiveis.html', context)

#Views das Ferramentas
@staff_member_required
def ferramentas(request):
    return render(request,'erp/ferramentas.html')

@staff_member_required
def mass_import(request):
    import threading
    import zipfile
    import os
    result={'erro':[],'ok':[],'repetido':[],'img_grande':[]}
    def processo():
        qtd_jogos=30 #quantidade de jogos que será feita por grupo
        n=0
        extraction_path = os.path.join('mediafiles','massimport')
        nonlocal result
        #
        if request.method == "POST":
            #Uploads file sent in post
            file = request.FILES['file']
            url = handle_file_upload(file, extraction_path)
            #Unzip file:
            path = os.path.join('mediafiles','massimport')
            file=os.listdir(path)
            if not zipfile.is_zipfile(os.path.join(path,file[0])): #Verificar se o arquivo é .zip
                messages.error(request,f'Arquivo enviado não é .zip')
                for filename in os.listdir(extraction_path):
                    os.remove(os.path.join(extraction_path, filename))
                return render(request,'erp/mass-import.html')
            with zipfile.ZipFile(os.path.join(path,file[0])) as zip_file:
                for zip_info in zip_file.infolist():
                    if zip_info.filename.lower().endswith(('.png','.jpg','.jpeg','.webp','.csv')):
                        with zip_file.open(zip_info) as source,open(os.path.join(extraction_path,os.path.basename(zip_info.filename)),'wb') as target:
                            target.write(source.read())
            #Process files:
            while True: 
                csv,n=ler_csv(n,qtd_jogos)
                #print(csv)
                #break
                if len(csv['nome_jogo'])==0: #Verificar se a lista vier vazia
                    break
                #Trabalhar com cada jogo do csv parcial
                for i in range(len(csv['nome_jogo_ludopedia'])):
                    if Estoque.objects.filter(nomeludopedia=csv['nome_jogo_ludopedia'][i]).exists(): # verificando se tem jogo no banco
                        result['repetido'].append(csv['nome_jogo'][i])
                        continue
                    dados = buscar_informacoes(csv['nome_jogo_ludopedia'][i])
                    if dados == 'error(naoencontrado)':
                        result['erro'].append(csv['nome_jogo'][i])
                        continue
                    #Criando jogo
                    print(csv['nome_jogo'][i])
                    jogo_novo=Estoque()
                    jogo_novo.nome=csv['nome_jogo'][i]
                    jogo_novo.nomeludopedia=csv['nome_jogo_ludopedia'][i]
                    if dados[4][:1]=='\n':
                        jogo_novo.descricao = dados[4][1:]
                    else:
                        jogo_novo.descricao = dados[4]
                    jogo_novo.valor=csv['vlr_aluguel'][i]
                    jogo_novo.quantidade = csv['qtd_estoque'][i]
                    jogo_novo.tempodejogo = int(dados[2].split()[0])
                    list_jogadores=dados[3].strip().split(' ')
                    if len(list_jogadores)>=4:
                        jogo_novo.minjogadores = int(dados[3].split(' ')[0])
                        jogo_novo.maxjogadores = int(dados[3].split(' ')[2])
                    else:
                        jogo_novo.minjogadores = int(dados[3].split(' ')[0])
                        jogo_novo.maxjogadores = 0
                    jogo_novo.idade = dados[1][6:]
                    jogo_novo.save()
                    #Criando as tags
                    tag_list=csv['tags'][i].split(":")
                    for tag_csv in tag_list:
                        try:
                            tag=Tag.objects.get(tag=tag_csv)
                        except:
                            tag = Tag.objects.create(tag=tag_csv)
                            tag.save()
                        jogo_novo.tags.add(tag)
                        jogo_novo.save()
                    #Criando imagens
                    imagens=[imagem for imagem in os.listdir(os.path.join('mediafiles','massimport')) if csv['nome_jogo_ludopedia'][i] in imagem]
                    for imagem in imagens:
                        if os.path.getsize(os.path.join(extraction_path,os.path.basename(imagem)))<200001:
                            with open(os.path.join(extraction_path,os.path.basename(imagem)),'rb') as f:
                                image_file = File(f)
                                image_file.name = os.path.basename(f.name)
                                id_jogo = Estoque.objects.get(pk=jogo_novo.pk)
                                image = Imagemestoque()
                                image.produto = id_jogo
                                image.foto = image_file
                                image.save()
                        else:
                            result['img_grande'].append(csv['nome_jogo'][i])
                    result['ok'].append(csv['nome_jogo'][i])
                if len(csv['nome_jogo'])<qtd_jogos:
                    break
            for filename in os.listdir(extraction_path):
                os.remove(os.path.join(extraction_path, filename))
            #envio da mensagem de resultado
            verificacao=0
            if len(result['erro'])>0:
                verificacao+=1
            if len(result['ok'])>0:
                verificacao+=1
            if len(result['repetido'])>0:
                verificacao+=1
            if verificacao>=2:
                if len(result['ok'])>0:
                    texto=''
                    for jogo in result['ok']:
                        texto+=jogo+','
                    texto=texto[:-2]
                    messages.success(request,f'''Processo Finalizado! Jogos que foram adicionados corretamente: {texto}''')
                if len(result['erro'])>0:
                    texto=''
                    for jogo in result['erro']:
                        texto+=jogo+','
                    texto=texto[:-2]
                    messages.error(request,f'''Jogos que não foram adicionados por estarem com o nome incorreto: {texto}''')
                if len(result['repetido'])>0:
                    texto=''
                    for jogo in result['repetido']:
                        texto+=jogo+','
                    texto=texto[:-2]
                    messages.warning(request,f'''Jogos que não foram adicionados por ja estarem registrados: {texto}''')
            elif result['erro']:
                messages.error(request,f'''Nenhum jogo foi adicionado corretamente! Verifique se os nomes de pesquisa estão corretos ou se o arquivo esta estruturado corretamente''')
            elif result['repetido']:
                messages.warning(request,f'''Nenhum jogo foi adicionado corretamente! Todos os jogos ja existem no banco de dados''')
            else:
                messages.success(request,'''Processo Finalizado! Todos os jogos foram adicionados corretamente com as informações inseridas e da Ludopedia''')
            if len(result['img_grande'])>0:
                texto=''
                for jogo in result['img_grande']:
                    texto+=jogo+','
                texto=texto[:-2]
                messages.warning(request,f'''Lista de imagens que eram maiores que 200kb: {texto}''')

    thread=threading.Thread(target=processo)
    thread.start()
    thread.join()

    return render(request,'erp/mass-import.html')
        
@staff_member_required
def carrossel_images(request):
    import zipfile
    import os
    extraction_path = os.path.join('erp','static','img','carrossel')
    if request.method == "POST":
        #Delete current files
        for filename in os.listdir(extraction_path):
            os.remove(os.path.join(extraction_path, filename))
        #Uploads file sent in post
        file = request.FILES['file']
        url = handle_file_upload(file, extraction_path)
        #Unzip file
        file=os.listdir(extraction_path)
        if not zipfile.is_zipfile(os.path.join(extraction_path,file[0])): #Verificar se o arquivo é .zip
            messages.error(request,f'Arquivo enviado não é .zip')
            for filename in os.listdir(extraction_path):
                os.remove(os.path.join(extraction_path, filename))
            return
        with zipfile.ZipFile(os.path.join(extraction_path,file[0])) as zip_file:
            zip_list=zip_file.infolist()
            img_files=[f for f in zip_list if f.filename.endswith(('.png','.jpg','.jpeg','.webp'))]
            if len(img_files) != 16:
                messages.error(request,'O arquivo .zip não possui 16 imagens!')
                return render(request,'erp/carrossel-images.html')
            for zip_info in img_files:
                if zip_info.filename.lower().endswith(('.png','.jpg','.jpeg','.webp')):
                    with zip_file.open(zip_info) as source,open(os.path.join(extraction_path,os.path.basename(zip_info.filename)),'wb') as target:
                        target.write(source.read())
        #Process files, verify file's size delete .zip file
        linha=1
        imagem=1
        for filename in os.listdir(extraction_path):
            if filename.endswith('.zip'):
                os.remove(os.path.join(extraction_path, filename))
            else:
                if os.path.getsize(os.path.join(extraction_path,filename))<200001:
                    os.rename(os.path.join(extraction_path,filename),
                            os.path.join(extraction_path,f'line_{linha}_photo_{imagem}{os.path.splitext(filename)[1]}'))
                    if imagem==8:
                        linha=2
                        imagem=1
                    else:
                        imagem+=1
                else:
                    messages.error(request,f'A imagem {filename} é maior do que 200kb. Substitua a imagem e reenvie o arquivo .zip')
                    for filename in os.listdir(extraction_path):
                        os.remove(os.path.join(extraction_path, filename))
                    return render(request,'erp/carrossel-images.html')
        messages.success(request,'Imagens do carrossel alteradas com sucesso!')
        return render(request,'erp/carrossel-images.html')
    else:
        return render(request,'erp/carrossel-images.html')
    
def login2(request):
    if request.method=="POST":
        username=request.POST['username']
        senha=request.POST['password']
        user=authenticate(request,username=username,password=senha)
        if user is not None:
            login(request,user)
            messages.success(request,("Login com Sucesso"))
            return redirect('inicio')
        else:
            messages.error(request,("Erro ao fazer login"))
            return redirect('login2')
    else:
        header_context = generate_template_header_context(request)
        #user_form = LoginForm(instance=request.user)
        user_form = LoginForm()
        context = {
            "header_context" : header_context,
            "form" : user_form,
        }
        return render(request, 'erp/login2.html', context)
